@@ -273,6 +273,7 @@ int main(int argc, char **argv)
 	}
 
 	for (i = 0; i < ImageHeight; i++) {
+	    /* coverity[uninit_use_in_call] */
 	    if (EGifPutLine(GifFile, Line, ImageWidth) == GIF_ERROR)
 		QuitGifError(GifFile);
 	    GifQprintf("\b\b\b\b%-4d", Count++);
@@ -323,8 +324,14 @@ int main(int argc, char **argv)
 	}
     }
 
-    if (EGifCloseFile(GifFile) == GIF_ERROR)
-	QuitGifError(GifFile);
+    if (EGifCloseFile(GifFile, &ErrorCode) == GIF_ERROR)
+    {
+	PrintGifError(ErrorCode);
+	if (GifFile != NULL) {
+	    EGifCloseFile(GifFile, NULL);
+	}
+	exit(EXIT_FAILURE);
+    }
 
     return 0;
 }
@@ -336,8 +343,9 @@ static void QuitGifError(GifFileType *GifFile)
 {
     if (GifFile != NULL) {
 	PrintGifError(GifFile->Error);
-	EGifCloseFile(GifFile);
+	EGifCloseFile(GifFile, NULL);
     }
+    exit(EXIT_FAILURE);
 }
 
 /* end */
